"""
    .. autoclass:: FootCiteRole
        :show-inheritance:

        .. automethod:: result_nodes
"""

from typing import TYPE_CHECKING, List, Tuple, cast

import docutils.nodes
from docutils.nodes import make_id
from pybtex.plugin import find_plugin
from sphinx.roles import XRefRole
from sphinx.util.logging import getLogger

from .style.referencing import format_references
from .style.template import FootReferenceInfo
from .transforms import node_text_transform

if TYPE_CHECKING:
    from sphinx.environment import BuildEnvironment

    from .domain import BibtexDomain
    from .foot_domain import BibtexFootDomain

logger = getLogger(__name__)


class FootCiteRole(XRefRole):
    """Class for processing the :rst:role:`footcite` role."""

    def result_nodes(
        self,
        document: "docutils.nodes.document",
        env: "BuildEnvironment",
        node: "docutils.nodes.Element",
        is_ref: bool,
    ) -> Tuple[List["docutils.nodes.Node"], List["docutils.nodes.system_message"]]:
        """Transform node into footnote references, and
        add footnotes to a node stored in the environment's temporary data
        if they are not yet present.

        .. seealso::

           The node containing all footnotes is inserted into the document by
           :meth:`.foot_directives.FootBibliographyDirective.run`.
        """
        if not node.get("refdomain"):
            assert node["reftype"] == "footcite"
            node["refdomain"] = "footcite"
            node["reftype"] = "p"
        foot_domain = cast("BibtexFootDomain", self.env.get_domain("footcite"))
        keys = [key.strip() for key in self.target.split(",")]  # type: ignore
        try:
            foot_bibliography = env.temp_data["bibtex_foot_bibliography"]
        except KeyError:
            env.temp_data[
                "bibtex_foot_bibliography"
            ] = foot_bibliography = foot_domain.bibliography_header.deepcopy()
        foot_old_refs = env.temp_data.setdefault("bibtex_foot_old_refs", set())
        foot_new_refs = env.temp_data.setdefault("bibtex_foot_new_refs", set())
        style = find_plugin(
            "pybtex.style.formatting", self.config.bibtex_default_style
        )()
        references = []
        domain = cast("BibtexDomain", self.env.get_domain("cite"))
        # count only incremented at directive, see foot_directives run method
        footbibliography_count = env.temp_data.setdefault(
            "bibtex_footbibliography_count", 0
        )
        footcite_names = env.temp_data.setdefault("bibtex_footcite_names", {})
        for key in keys:
            entry = domain.bibdata.data.entries.get(key)
            if entry is not None:
                formatted_entry = style.format_entry(label="", entry=entry)
                if key not in (foot_old_refs | foot_new_refs):
                    footnote = docutils.nodes.footnote(auto=1)
                    # no automatic ids for footnotes: force non-empty template
                    template: str = (
                        env.app.config.bibtex_footcite_id
                        if env.app.config.bibtex_footcite_id
                        else "footcite-{key}"
                    )
                    raw_id = template.format(
                        footbibliography_count=footbibliography_count + 1, key=entry.key
                    )
                    # format name with make_id for consistency with cite role
                    name = make_id(raw_id)
                    footnote["names"] += [name]
                    footcite_names[entry.key] = name
                    footnote += domain.backend.paragraph(formatted_entry)
                    document.note_autofootnote(footnote)
                    document.note_explicit_target(footnote, footnote)
                    node_text_transform(footnote)
                    foot_bibliography += footnote
                    foot_new_refs.add(key)
                references.append(
                    (
                        entry,
                        formatted_entry,
                        FootReferenceInfo(
                            key=entry.key,
                            refname=footcite_names[entry.key],
                            document=document,
                        ),
                    )
                )
            else:
                logger.warning(
                    'could not find bibtex key "%s"' % key,
                    location=(env.docname, self.lineno),
                    type="bibtex",
                    subtype="key_not_found",
                )
        ref_nodes = format_references(
            foot_domain.reference_style, node["reftype"], references
        ).render(domain.backend)
        return ref_nodes, []



from typing import TYPE_CHECKING, Any, Dict, Iterable, List

import pybtex.plugin
import pytest
from pybtex.database import Entry, Person
from pybtex.richtext import HRef
from pybtex.style.template import FieldIsMissing, Node, _format_list

# direct import of the plugin to ensure we are testing this specific class
from sphinxcontrib.bibtex.richtext import BaseReferenceText
from sphinxcontrib.bibtex.style.names.last import LastNameStyle
from sphinxcontrib.bibtex.style.referencing import BaseReferenceStyle, format_references
from sphinxcontrib.bibtex.style.referencing.basic_author_year import (
    BasicAuthorYearTextualReferenceStyle,
)
from sphinxcontrib.bibtex.style.referencing.basic_super import (
    BasicSuperParentheticalReferenceStyle,
    BasicSuperTextualReferenceStyle,
)
from sphinxcontrib.bibtex.style.template import entry_label, join, names, node

if TYPE_CHECKING:
    from pybtex.backends import BaseBackend
    from pybtex.richtext import BaseText
    from pybtex.style.formatting import BaseStyle


def test_style_names_last() -> None:
    name = Person(string=r"Charles Louis Xavier Joseph de la Vall{\'e}e Poussin")
    last = LastNameStyle().format
    assert last(name).format().render_as("latex") == "de~la Vall{é}e~Poussin"
    assert (
        last(name).format().render_as("html")
        == 'de&nbsp;la Vall<span class="bibtex-protected">é</span>e&nbsp;Poussin'
    )
    name2 = Person(first="First", last="Last", middle="Middle")
    assert last(name2).format().render_as("latex") == "Last"


def test_style_names() -> None:
    auth = Person("First Last")
    fields = dict(title="The Book", year="2000", publisher="Santa")
    entry = Entry(type_="book", fields=fields, persons=dict(author=[auth]))
    style = BasicAuthorYearTextualReferenceStyle()
    rich_name = style.person.names("author", full=True).format_data(
        dict(entry=entry, style=style)
    )
    assert rich_name.render_as("text") == "Last"


def test_style_names_no_author() -> None:
    entry = Entry(type_="book")
    with pytest.raises(FieldIsMissing):
        names("author").format_data(dict(entry=entry))


class SimpleReferenceText(BaseReferenceText[str]):
    """A simple reference text class storing the url as a string in *info*."""

    def render(self, backend):
        url = self.info[0]
        return HRef(url, *self.parts).render(backend)


@node
def simple_reference(children, data: Dict[str, Any]):
    """Pybtex node for inserting a docutils reference node to a citation.
    The children of the node
    comprise the content of the reference, and any referencing information
    is stored in the *reference_info* key of the *data*.
    The data must also contain a *style* key pointing to the corresponding
    :class:`~sphinxcontrib.bibtex.style.referencing.BaseReferenceStyle`.
    """
    parts = _format_list(children, data)
    info = data["reference_info"]
    return SimpleReferenceText(info, *parts)


class SimpleReferenceStyle(BaseReferenceStyle):
    def role_names(self) -> Iterable[str]:
        return ["p"]

    def outer(self, role_name: str, children: List["BaseText"]) -> "Node":
        return join["{", join(";")[children], "}"]

    def inner(self, role_name: str) -> "Node":
        return simple_reference[entry_label]


def test_simple_reference_style() -> None:
    cit_style: "BaseStyle" = pybtex.plugin.find_plugin(
        "pybtex.style.formatting", "unsrtalpha"
    )()
    ref_style = SimpleReferenceStyle()
    auth1 = Person("First Last")
    auth2 = Person("Ein Zwei")
    auth3 = Person("Primo Secundo")
    fields1 = dict(title="The Book", year="2000", publisher="Santa")
    entry1 = Entry(type_="book", fields=fields1, persons=dict(author=[auth1]))
    entry2 = Entry(type_="book", fields=fields1, persons=dict(author=[auth2]))
    entry3 = Entry(type_="book", fields=fields1, persons=dict(author=[auth3]))
    entries = [entry1, entry2, entry3]
    formatted_entries = list(cit_style.format_entries(entries))
    infos = ["#id1", "#id2", "#id3"]
    references = list(zip(entries, formatted_entries, infos))
    backend: "BaseBackend" = pybtex.plugin.find_plugin("pybtex.backends", "html")()
    assert "p" in ref_style.role_names()
    assert (
        format_references(ref_style, "p", references).render(backend)
        == '{<a href="#id1">Las00</a>;<a href="#id2">Zwe00</a>'
        ';<a href="#id3">Sec00</a>}'
    )


# need to instantiate these for 100% coverage
def test_super_coverage() -> None:
    BasicSuperParentheticalReferenceStyle()
    BasicSuperTextualReferenceStyle()



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Iterable, List, Union

from sphinxcontrib.bibtex.style.template import (
    join,
    join2,
    post_text,
    pre_text,
    reference,
    year,
)

from . import BaseReferenceStyle, BracketStyle, PersonStyle

if TYPE_CHECKING:
    from pybtex.richtext import BaseText
    from pybtex.style.template import Node


@dataclass
class BasicAuthorYearParentheticalReferenceStyle(BaseReferenceStyle):
    """Parenthetical reference by author-year."""

    #: Bracket style.
    bracket: BracketStyle = field(default_factory=BracketStyle)

    #: Person style.
    person: PersonStyle = field(default_factory=PersonStyle)

    #: Separator between author and year.
    author_year_sep: Union["BaseText", str] = ", "

    #: Separator between pre-text and citation.
    pre_text_sep: Union["BaseText", str] = " "

    #: Separator between citation and post-text.
    post_text_sep: Union["BaseText", str] = ", "

    def role_names(self) -> Iterable[str]:
        return [
            f"{alt}p{full_author}" for alt in ["", "al"] for full_author in ["", "s"]
        ]

    def outer(self, role_name: str, children: List["BaseText"]) -> "Node":
        return self.bracket.outer(
            children, brackets="al" not in role_name, capfirst=False
        )

    def inner(self, role_name: str) -> "Node":
        return join2(sep1=self.pre_text_sep, sep2=self.post_text_sep)[
            pre_text,
            reference[
                join(sep=self.author_year_sep)[
                    self.person.author_or_editor_or_title(full="s" in role_name),
                    year,
                ]
            ],
            post_text,
        ]


@dataclass
class BasicAuthorYearTextualReferenceStyle(BaseReferenceStyle):
    """Textual reference by author-year."""

    #: Bracket style.
    bracket: BracketStyle = field(default_factory=BracketStyle)

    #: Person style.
    person: PersonStyle = field(default_factory=PersonStyle)

    #: Separator between text and reference.
    text_reference_sep: Union["BaseText", str] = " "

    #: Separator between pre-text and citation.
    pre_text_sep: Union["BaseText", str] = " "

    #: Separator between citation and post-text.
    post_text_sep: Union["BaseText", str] = ", "

    def role_names(self) -> Iterable[str]:
        return [
            f"{capfirst}t{full_author}"
            for capfirst in ["", "c"]
            for full_author in ["", "s"]
        ]

    def outer(self, role_name: str, children: List["BaseText"]) -> "Node":
        return self.bracket.outer(children, brackets=False, capfirst="c" in role_name)

    def inner(self, role_name: str) -> "Node":
        return join(sep=self.text_reference_sep)[
            self.person.author_or_editor_or_title(full="s" in role_name),
            join[
                self.bracket.left,
                join2(sep1=self.pre_text_sep, sep2=self.post_text_sep)[
                    pre_text,
                    reference[year],
                    post_text,
                ],
                self.bracket.right,
            ],
        ]



"""
    .. autoclass:: CitationRef
        :members:

    .. autoclass:: CiteRole
        :show-inheritance:

        .. automethod:: result_nodes
"""

from typing import TYPE_CHECKING, List, NamedTuple, cast

import docutils.nodes
from pybtex.plugin import find_plugin
from sphinx.roles import XRefRole

from .citation_target import CitationTarget, parse_citation_targets

if TYPE_CHECKING:
    from .domain import BibtexDomain


class CitationRef(NamedTuple):
    """Information about a citation reference."""

    citation_ref_id: str  #: Unique id of this citation reference.
    docname: str  #: Document name.
    line: int  #: Line number.
    targets: List[CitationTarget]  #: Citation targets (key, pre, post).


class CiteRole(XRefRole):

    """Class for processing the :rst:role:`cite` role."""

    backend = find_plugin("pybtex.backends", "docutils")()
    innernodeclass = docutils.nodes.inline

    def result_nodes(self, document, env, node, is_ref):
        """Associate the pending_xref with the cite domain,
        and note the cited citation keys.
        """
        if not node.get("refdomain"):
            assert node["reftype"] == "cite"
            node["refdomain"] = "cite"
            node["reftype"] = "p"
        document.note_explicit_target(node, node)  # for backrefs
        domain = cast("BibtexDomain", env.get_domain("cite"))
        domain.citation_refs.append(
            CitationRef(
                citation_ref_id=node["ids"][0],
                docname=env.docname,
                line=document.line,
                targets=list(parse_citation_targets(self.target)),
            )
        )
        return [node], []



"""
    .. autoclass:: BibliographyKey
        :members:

    .. autoclass:: BibliographyValue
        :members:

    .. autoclass:: BibliographyDirective

        .. automethod:: run
"""

import ast  # parse(), used for filter
from typing import TYPE_CHECKING, Dict, List, NamedTuple, cast

import docutils.nodes
import docutils.parsers.rst.directives as directives
import sphinx.util
from docutils.parsers.rst import Directive

from .bibfile import _make_ids, normpath_filename
from .nodes import bibliography as bibliography_node

if TYPE_CHECKING:
    from sphinx.environment import BuildEnvironment

    from .domain import BibtexDomain


logger = sphinx.util.logging.getLogger(__name__)


class BibliographyKey(NamedTuple):
    """Unique key for each bibliography directive."""

    docname: str  #: Name of the document where the bibliography resides.
    id_: str  #: The id of the bibliography node in the document.


class BibliographyValue(NamedTuple):
    """Contains information about a bibliography directive."""

    line: int  #: Line number of the directive in the document.
    bibfiles: List[str]  #: List of bib files for this directive.
    style: str  #: The pybtex style.
    list_: str  #: The list type.
    enumtype: str  #: The sequence type (for enumerated lists).
    start: int  #: The start of the sequence (for enumerated lists).
    labelprefix: str  #: String prefix for pybtex generated labels.
    keyprefix: str  #: String prefix for citation keys.
    filter_: ast.AST  #: Parsed filter expression.
    citation_nodes: Dict[str, docutils.nodes.Element]  #: key -> citation node
    keys: List[str]  #: Keys listed as content of the directive.


class BibliographyDirective(Directive):

    """Class for processing the :rst:dir:`bibliography` directive.

    Produces a
    :class:`~sphinxcontrib.bibtex.nodes.bibliography` node,
    along with (empty) citation nodes that will be formatted later in the
    *env-updated* stage, and inserted into the document in a post-transform.
    We cannot insert the citation nodes here because we do not yet know
    which keys have been cited.

    .. seealso::

       Further processing of the resulting
       :class:`~sphinxcontrib.bibtex.nodes.bibliography` node is done
       by
       :class:`~sphinxcontrib.bibtex.transforms.BibliographyTransform`.
    """

    required_arguments = 0
    optional_arguments = 1
    final_argument_whitespace = True
    has_content = True
    option_spec = {
        "cited": directives.flag,
        "notcited": directives.flag,
        "all": directives.flag,
        "filter": directives.unchanged,
        "style": directives.unchanged,
        "list": directives.unchanged,
        "enumtype": directives.unchanged,
        "start": (
            lambda value: directives.positive_int(value) if value != "continue" else -1
        ),
        "labelprefix": directives.unchanged,
        "keyprefix": directives.unchanged,
    }

    def _get_filter(self):
        """Get parsed filter from options."""
        env = cast("BuildEnvironment", self.state.document.settings.env)
        if "filter" in self.options:
            if "all" in self.options:
                logger.warning(
                    ":filter: overrides :all:",
                    location=(env.docname, self.lineno),
                    type="bibtex",
                    subtype="filter_overrides",
                )
            if "notcited" in self.options:
                logger.warning(
                    ":filter: overrides :notcited:",
                    location=(env.docname, self.lineno),
                    type="bibtex",
                    subtype="filter_overrides",
                )
            if "cited" in self.options:
                logger.warning(
                    ":filter: overrides :cited:",
                    location=(env.docname, self.lineno),
                    type="bibtex",
                    subtype="filter_overrides",
                )
            try:
                return ast.parse(self.options["filter"])
            except SyntaxError:
                logger.warning(
                    "syntax error in :filter: expression"
                    + " ("
                    + self.options["filter"]
                    + "); "
                    "the option will be ignored",
                    location=(env.docname, self.lineno),
                    type="bibtex",
                    subtype="filter_syntax_error",
                )
                return ast.parse("cited")
        elif "all" in self.options:
            return ast.parse("True")
        elif "notcited" in self.options:
            return ast.parse("not cited")
        else:
            # the default filter: include only cited entries
            return ast.parse("cited")

    def run(self):
        """Process .bib files, set file dependencies, and create a
        node that is to be transformed to the entries of the
        bibliography.
        """
        env = cast("BuildEnvironment", self.state.document.settings.env)
        domain = cast("BibtexDomain", env.get_domain("cite"))
        filter_ = self._get_filter()
        if self.arguments:
            bibfiles = []
            for bibfile in self.arguments[0].split():
                normbibfile = normpath_filename(env, bibfile)
                if normbibfile not in domain.bibdata.bibfiles:
                    logger.warning(
                        "{0} not found or not configured"
                        " in bibtex_bibfiles".format(bibfile),
                        location=(env.docname, self.lineno),
                        type="bibtex",
                        subtype="bibfile_error",
                    )
                else:
                    bibfiles.append(normbibfile)
        else:
            bibfiles = list(domain.bibdata.bibfiles.keys())
        for bibfile in bibfiles:
            env.note_dependency(bibfile)
        # generate nodes and ids
        keyprefix: str = self.options.get("keyprefix", "")
        list_: str = self.options.get("list", "citation")
        if list_ not in {"bullet", "enumerated", "citation"}:
            logger.warning(
                "unknown bibliography list type '{0}'.".format(list_),
                location=(env.docname, self.lineno),
                type="bibtex",
                subtype="list_type_error",
            )
            list_ = "citation"
        if list_ in {"bullet", "enumerated"}:
            citation_node_class = docutils.nodes.list_item
        else:
            citation_node_class = docutils.nodes.citation
        bibliography_count = env.temp_data["bibtex_bibliography_count"] = (
            env.temp_data.get("bibtex_bibliography_count", 0) + 1
        )
        ids = set(self.state.document.ids.keys())
        node = bibliography_node(
            "",
            docname=env.docname,
            ids=_make_ids(
                docname=env.docname,
                lineno=self.lineno,
                ids=ids,
                raw_id=env.app.config.bibtex_bibliography_id.format(
                    bibliography_count=bibliography_count
                ),
            ),
        )
        self.state.document.note_explicit_target(node, node)
        # we only know which citations to included at resolve stage
        # but we need to know their ids before resolve stage
        # so for now we generate a node, and thus, an id, for every entry
        citation_nodes: Dict[str, docutils.nodes.Element] = {
            keyprefix
            + entry.key: citation_node_class(
                ids=_make_ids(
                    docname=env.docname,
                    lineno=self.lineno,
                    ids=ids,
                    raw_id=env.app.config.bibtex_cite_id.format(
                        bibliography_count=bibliography_count, key=keyprefix + entry.key
                    ),
                )
            )
            for entry in domain.get_entries(bibfiles)
        }
        for citation_node in citation_nodes.values():
            self.state.document.note_explicit_target(citation_node, citation_node)
        # check and get keys
        keys = []
        for key in self.content:
            if keyprefix + key not in citation_nodes:
                logger.warning(
                    'could not find bibtex key "%s"' % key,
                    location=(env.docname, self.lineno),
                    type="bibtex",
                    subtype="key_not_found",
                )
            else:
                keys.append(key)
        # create bibliography object
        bibliography = BibliographyValue(
            line=self.lineno,
            list_=list_,
            enumtype=self.options.get("enumtype", "arabic"),
            start=self.options.get("start", 1),
            style=self.options.get("style", env.app.config.bibtex_default_style),
            filter_=filter_,
            labelprefix=self.options.get("labelprefix", ""),
            keyprefix=keyprefix,
            bibfiles=bibfiles,
            citation_nodes=citation_nodes,
            keys=keys,
        )
        bib_key = BibliographyKey(docname=env.docname, id_=node["ids"][0])
        assert bib_key not in domain.bibliographies
        domain.bibliographies[bib_key] = bibliography
        return [node]



"""
    test_natbib
    ~~~~~~~~~~~

    Test the natbib extension, which serves as an example for comparison.
"""

import pytest


@pytest.mark.sphinx("html", testroot="natbib")
def test_natbib(app, warning) -> None:
    app.build()
    assert not warning.getvalue()


@pytest.mark.sphinx("latex", testroot="natbib")
def test_natbib_latex(app, warning) -> None:
    app.build()
    assert not warning.getvalue()


@pytest.mark.sphinx("html", testroot="natbib_keynotfound")
def test_natbib_keynotfound(app, warning) -> None:
    app.build()
    warning.seek(0)
    warnings = warning.readlines()
    assert len(warnings) == 1
    assert "WARNING: cite-key `XXX` not found in bibtex file" in warnings[0]


@pytest.mark.sphinx("html", testroot="natbib_norefs")
def test_natbib_norefs(app, warning) -> None:
    app.build()
    warning.seek(0)
    warnings = warning.readlines()
    assert len(warnings) == 1
    assert "WARNING: no `refs` directive found" in warnings[0]


def test_natbib_citation_transform_str_repr() -> None:
    from test.natbib import DEFAULT_CONF, CitationTransform

    from pybtex.database import Entry

    ref = Entry(type_="misc")
    ref.key = "somekey"
    node = CitationTransform(
        pre="",
        post="",
        typ="cite:p",
        global_keys={},
        config=DEFAULT_CONF.copy(),
        refs=[ref],
    )
    assert str(node) == "somekey"
    assert repr(node) == "<somekey>"


@pytest.mark.sphinx("text", testroot="natbib_conf")
def test_natbib_conf(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.txt").read_text()
    assert "{One, 2001/ Two, 2002}" in output



import re

needs_sphinx = "2.1"
extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
]
master_doc = "index"
project = "sphinxcontrib-bibtex"
copyright = "2011-2023, Matthias C. M. Troffaes"
with open("../src/sphinxcontrib/bibtex/__init__.py", "rt") as version_file:
    release = re.search('"version": "(.+)"', version_file.read()).group(1)
version = ".".join(release.split(".")[:2])
exclude_patterns = ["_build"]
pygments_style = "sphinx"
intersphinx_mapping = {
    "python": ("https://docs.python.org/3/", None),
    "sphinx": ("http://www.sphinx-doc.org/en/master/", None),
}
html_theme = "alabaster"



from test.common import (
    html_citation_refs_single,
    html_citations,
    html_footnote_refs,
    html_footnotes,
)

import pytest


@pytest.mark.numpydoc
@pytest.mark.sphinx(
    "html",
    testroot="autodoc",
    confoverrides={
        "extensions": ["sphinxcontrib.bibtex", "numpydoc"],
        "numpydoc_class_members_toctree": False,
    },
)
def test_numpydoc(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "doc_cite.html").read_text()
    labels = [
        "One",
        "Two",
        "Thr",
        "Fou",
        "Fiv",
        "Six",
        "Sev",
        "Eig",
        "Nin",
        "Ten",
        "Ele",
    ]
    titles = [
        "Een",
        "Twee",
        "Drie",
        "Vier",
        "Vijf",
        "Zes",
        "Zeven",
        "Acht",
        "Negen",
        "Tien",
        "Elf",
    ]
    for label, title in zip(labels, titles):
        assert len(html_citation_refs_single(label=label).findall(output)) == 1
        assert len(html_citations(label=label).findall(output)) == 1
        match_ref = html_citation_refs_single(label=label).search(output)
        match = html_citations(label=label).search(output)
        assert match_ref
        assert match
        assert match_ref.group("refid") == match.group("id_")
        assert title in match.group("text")
        assert match_ref.group("id_") == match.group("backref")
    output2 = (app.outdir / "doc_footcite.html").read_text()
    assert len(html_footnote_refs().findall(output2)) == 11
    for title in titles:
        text = ".*" + title + ".*"
        assert len(html_footnotes(text=text).findall(output2)) == 1
        match = html_footnotes(text=text).search(output2)
        assert match
        id_ = match.group("id_")
        assert len(html_footnote_refs(refid=id_).findall(output2)) == 1



"""
    Domain for footnote citations.

    .. autoclass:: BibtexFootDomain
        :members:
"""

from typing import TYPE_CHECKING, Dict, List, Tuple

import docutils.nodes
import docutils.utils
import sphinx.util
from sphinx.domains import Domain, ObjType
from sphinx.locale import _

import sphinxcontrib.bibtex.plugin

from .domain import parse_header
from .foot_roles import FootCiteRole
from .style.referencing import BaseReferenceStyle

if TYPE_CHECKING:
    from sphinx.addnodes import pending_xref
    from sphinx.builders import Builder
    from sphinx.environment import BuildEnvironment

logger = sphinx.util.logging.getLogger(__name__)


class BibtexFootDomain(Domain):
    """Sphinx domain for footnote citations."""

    name = "footcite"
    label = "BibTeX Footnote Citations"
    data_version = 0
    initial_data = dict(
        bibliography_header=docutils.nodes.container(),
    )
    reference_style: BaseReferenceStyle

    @property
    def bibliography_header(self) -> docutils.nodes.Element:
        return self.data["bibliography_header"]

    def __init__(self, env: "BuildEnvironment"):
        # set up referencing style
        style = sphinxcontrib.bibtex.plugin.find_plugin(
            "sphinxcontrib.bibtex.style.referencing",
            env.app.config.bibtex_foot_reference_style,
        )
        self.reference_style = style()
        # set up object types and roles for referencing style
        role_names = self.reference_style.role_names()
        self.object_types = dict(
            citation=ObjType(_("citation"), *role_names, searchprio=-1),
        )
        self.roles = dict((name, FootCiteRole()) for name in role_names)
        # initialize the domain
        super().__init__(env)
        # parse bibliography header
        header = getattr(env.app.config, "bibtex_footbibliography_header")
        if header:
            self.data["bibliography_header"] = docutils.nodes.container()
            self.data["bibliography_header"] += parse_header(
                header, "foot_bibliography_header"
            )

    def merge_domaindata(self, docnames: List[str], otherdata: Dict) -> None:
        """Merge in data regarding *docnames* from domain data
        inventory *otherdata*.

        As there is no document specific data for this domain, this function
        does nothing.
        """
        pass

    def resolve_any_xref(
        self,
        env: "BuildEnvironment",
        fromdocname: str,
        builder: "Builder",
        target: str,
        node: "pending_xref",
        contnode: docutils.nodes.Element,
    ) -> List[Tuple[str, docutils.nodes.Element]]:
        """Resolve the pending reference *node* with the given *target*,
        where the reference comes from an "any" role.

        Since citation references are resolved to regular citations,
        and not to footnote citations,
        this implementation simply returns an empty list.
        """
        return []



import pytest
import sphinx


@pytest.mark.skipif(
    sphinx.version_info < (4, 0),
    reason="root_doc only supported in sphinx 4.0 and higher",
)
@pytest.mark.sphinx("pseudoxml", testroot="root_doc")
def test_root_doc(app, warning):
    app.build()
    assert not warning.getvalue()
    assert (app.outdir / "root.pseudoxml").exists()



"""
    .. autoclass:: bibliography
    .. autoclass:: raw_latex
    .. autofunction:: visit_raw_latex
    .. autofunction:: depart_raw_latex
"""

from docutils import nodes
from sphinx.writers.latex import LaTeXTranslator


class bibliography(nodes.General, nodes.Element):
    """Node for representing a bibliography. Replaced by a list of
    citations by
    :class:`~sphinxcontrib.bibtex.transforms.BibliographyTransform`.
    """

    pass


class raw_latex(
    nodes.Special, nodes.Inline, nodes.PreBibliographic, nodes.FixedTextElement
):
    """Node for representing raw latex data."""

    pass


def visit_raw_latex(self: LaTeXTranslator, node: raw_latex):
    """Called when entering a raw_latex node. Appends the node's raw source
    to the latex body.
    """
    self.body.append(node.rawsource)


def depart_raw_latex(self: LaTeXTranslator, node: raw_latex):
    """Called when leaving a raw_latex node."""
    pass



from Cython.Build import cythonize
from setuptools import find_packages, setup

setup(
    name="some-cython-module",
    version="0.0.1",
    license="BSD",
    author="Matthias C. M. Troffaes",
    author_email="matthias.troffaes@gmail.com",
    description="Test module for bibtex citations in cython modules",
    zip_safe=False,
    platforms="any",
    packages=find_packages("src"),
    package_dir={"": "src"},
    ext_modules=cythonize(
        ["src/some_cython_module/cite.pyx", "src/some_cython_module/footcite.pyx"]
    ),
    python_requires=">=3.6",
)



import pytest
import sphinx

if sphinx.version_info >= (7, 2):
    from pathlib import Path

    _rootdir = Path(__file__).parent.resolve() / "roots"
else:
    from sphinx.testing.path import path as Path  # type: ignore

    _rootdir = Path(__file__).parent.abspath() / "roots"  # type: ignore

pytest_plugins = "sphinx.testing.fixtures"
collect_ignore = ["roots"]


@pytest.fixture(scope="session")
def rootdir() -> Path:
    return _rootdir



import io
import re
from typing import Optional

from setuptools import find_namespace_packages, setup


def readfile(filename: str) -> str:
    with io.open(filename, encoding="utf-8") as stream:
        return stream.read()


readme = readfile("README.rst").split("\n")[5:]  # skip title and badges
requires = readfile("requirements.txt").split("\n")
version_match = re.search(
    '"version": "(.+)"', readfile("src/sphinxcontrib/bibtex/__init__.py")
)
assert version_match is not None, "version not found"
version = version_match.group(1)


# make entry point specifications
def plugin(plugin_name: str, mod_name: Optional[str] = None) -> str:
    if mod_name is None:
        mod_name = plugin_name
    path = "sphinxcontrib.bibtex.style.referencing"
    class_name = "".join(part.capitalize() for part in plugin_name.split("_"))
    return f"{plugin_name} = {path}.{mod_name}:{class_name}ReferenceStyle"


setup(
    name="sphinxcontrib-bibtex",
    version=version,
    url="https://github.com/mcmtroffaes/sphinxcontrib-bibtex",
    download_url="https://pypi.python.org/pypi/sphinxcontrib-bibtex",
    license="BSD",
    author="Matthias C. M. Troffaes",
    author_email="matthias.troffaes@gmail.com",
    description=readme[0],
    long_description="\n".join(readme[2:]),
    long_description_content_type="text/x-rst",
    zip_safe=False,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Documentation",
        "Topic :: Utilities",
    ],
    platforms="any",
    packages=find_namespace_packages("src"),
    package_dir={"": "src"},
    package_data={"sphinxcontrib.bibtex": ["py.typed"]},
    include_package_data=True,
    python_requires=">=3.7",
    install_requires=requires,
    tests_require=["pytest", "pytest-cov"],
    namespace_packages=["sphinxcontrib"],
    entry_points={
        "pybtex.style.names": [
            "last = sphinxcontrib.bibtex.style.names.last:LastNameStyle",
        ],
        "sphinxcontrib.bibtex.style.referencing": [
            plugin("author_year"),
            plugin("label"),
            plugin("super", "super_"),
            plugin("foot"),
        ],
    },
)



"""Some tests purely used for stepping into the debugger
to help understand what docutils/sphinx are doing.
"""

import shutil

import pytest

docutils_citation_xml = """
    <paragraph>
        <reference ids="id1" internal="True" refid="label">
            <inline>
                [Label]
    <citation backrefs="id1" docname="index" ids="label" names="label">
        <label support_smartquotes="False">
            Label
        <paragraph>
            The title.
"""

bibtex_citation_xml = """
    <paragraph>
        <inline ids="id1">
            [
            <reference internal="True" refid="id3" reftitle="The title.">
                tes
            ]
    <container ids="id2">
        <citation backrefs="id1" docname="index" ids="id3">
            <label support_smartquotes="False">
                tes
            <paragraph>
                The title.
"""


@pytest.mark.sphinx("pseudoxml", testroot="debug_docutils_citation")
def test_debug_docutils_citation(app, warning):
    """A simple test with a single standard docutils citation."""
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.pseudoxml").read_text()
    assert output.split("\n")[1:] == docutils_citation_xml.split("\n")[1:]


@pytest.mark.sphinx("pseudoxml", testroot="debug_bibtex_citation")
def test_debug_bibtex_citation(app, warning):
    """A simple test with a single standard docutils citation."""
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.pseudoxml").read_text()
    assert output.split("\n")[1:] == bibtex_citation_xml.split("\n")[1:]


# see issue 226
@pytest.mark.sphinx("pseudoxml", testroot="debug_bibtex_citation")
def test_rebuild_empty_outdir(make_app, app_params):
    args, kwargs = app_params
    app0 = make_app(freshenv=True, *args, **kwargs)
    app0.build()
    assert not app0._warning.getvalue()
    shutil.rmtree(app0.outdir)
    app1 = make_app(freshenv=False, *args, **kwargs)
    app1.build()
    assert "could not find bibtex key" not in app1._warning.getvalue()


@pytest.mark.sphinx("pseudoxml", testroot="debug_minimal_example")
def test_debug_minimal_example(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.pseudoxml").read_text()
    assert [line for line in output.split("\n")][1:] == [
        "    <paragraph>",
        "        See ",
        '        <inline ids="id1">',
        "            Nelson [",
        '            <reference internal="True" refid="id4" '
        'reftitle="Edward Nelson. Radically Elementary Probability Theory. '
        'Princeton University Press, 1987.">',
        "                Nel87",
        "            ]",
        "         for an introduction to non-standard analysis.",
        "        Non-standard analysis is fun ",
        '        <inline ids="id2">',
        "            [",
        '            <reference internal="True" refid="id4" '
        'reftitle="Edward Nelson. Radically Elementary Probability Theory. '
        'Princeton University Press, 1987.">',
        "                Nel87",
        "            ]",
        "        .",
        '    <container ids="id3">',
        '        <citation backrefs="id1 id2" docname="index" ids="id4">',
        '            <label support_smartquotes="False">',
        "                Nel87",
        "            <paragraph>",
        "                Edward Nelson.",
        "                 ",
        "                <emphasis>",
        "                    Radically Elementary Probability Theory",
        "                .",
        "                 ",
        "                Princeton University Press, 1987.",
        "",
    ]



import os
import sys

# viewcode extension specifically needed for this test
extensions = ["sphinxcontrib.bibtex", "sphinx.ext.viewcode", "sphinx.ext.autodoc"]
bibtex_bibfiles = []
# make sure we find the module
sys.path.insert(0, os.path.abspath("."))

exclude_patterns = ["_build"]



"""Test warnings on duplicate labels/keys."""

from test.common import html_citation_refs, html_citations

import pytest


@pytest.mark.sphinx("html", testroot="duplicate_label")
def test_duplicate_label(app, warning) -> None:
    # see github issue 14
    app.build()
    assert 'duplicate label "1" for keys "Test" and "Test2"' in warning.getvalue()
    output = (app.outdir / "doc1.html").read_text()
    output2 = (app.outdir / "doc2.html").read_text()
    assert html_citations(label="1").search(output)
    assert html_citations(label="1").search(output2)


@pytest.mark.sphinx("html", testroot="duplicate_citation")
def test_duplicate_citation(app, warning) -> None:
    app.build()
    warning.seek(0)
    warnings = list(warning.readlines())
    assert len(warnings) == 1
    assert 'duplicate citation for key "Test"' in warnings[0]
    # assure distinct citation ids
    output = (app.outdir / "index.html").read_text()
    ids = [match.group("id_") for match in html_citations().finditer(output)]
    assert len(ids) == 2  # just to check ids are found
    assert len(set(ids)) == 2, "citation ids not unique"


@pytest.mark.sphinx("html", testroot="duplicate_nearly_identical_entries")
def test_duplicate_nearly_identical_entries(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.html").read_text()
    cits = list(html_citations().finditer(output))
    cit_refs = list(html_citation_refs().finditer(output))
    assert len(cits) == len(cit_refs) == 2
    assert (
        {cit.group("label") for cit in cits}
        == {cit_ref.group("label") for cit_ref in cit_refs}
        == {"xyz19a", "xyz19b"}
    )


@pytest.mark.sphinx("html", testroot="duplicate_nearly_identical_keys", freshenv=True)
def test_duplicate_nearly_identical_keys_1(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.html").read_text()
    # assure both citations and citation references are present
    assert html_citation_refs(label="Smi").search(output)
    assert html_citation_refs(label="Pop").search(output)
    assert html_citation_refs(label="Ein").search(output)
    assert html_citations(label="Smi").search(output)
    assert html_citations(label="Pop").search(output)
    assert html_citations(label="Ein").search(output)
    # assure distinct ids for citations
    ids = {match.group("id_") for match in html_citations().finditer(output)}
    refids = {match.group("refid") for match in html_citation_refs().finditer(output)}
    assert None not in ids
    assert len(ids) == 3
    assert ids == refids


@pytest.mark.sphinx(
    "html",
    testroot="duplicate_nearly_identical_keys",
    freshenv=True,
    confoverrides={"bibtex_cite_id": "cite-{bibliography_count}-{key}"},
)
def test_duplicate_nearly_identical_keys_2(app, warning) -> None:
    app.build()
    warning.seek(0)
    warnings = list(warning.readlines())
    assert len(warnings) == 2
    assert "duplicate citation id cite-1-test" in warnings[0]
    assert "duplicate citation id cite-1-test" in warnings[1]
    output = (app.outdir / "index.html").read_text()
    # assure both citations and citation references are present
    assert html_citation_refs(label="Smi").search(output)
    assert html_citation_refs(label="Pop").search(output)
    assert html_citation_refs(label="Ein").search(output)
    assert html_citations(label="Smi").search(output)
    assert html_citations(label="Pop").search(output)
    assert html_citations(label="Ein").search(output)
    # assure distinct ids for citations
    ids = {match.group("id_") for match in html_citations().finditer(output)}
    refids = {match.group("refid") for match in html_citation_refs().finditer(output)}
    assert None not in ids
    assert len(ids) == 3
    assert ids == refids


# this test "accidentally" includes a user provided id which
# clashes with a bibtex generated citation id
@pytest.mark.sphinx("html", testroot="duplicate_citation_id")
def test_duplicate_citation_id(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.html").read_text()
    user_ids = {"id1", "id2", "id3"}
    ids = {match.group("id_") for match in html_citations().finditer(output)}
    refids = {match.group("refid") for match in html_citation_refs().finditer(output)}
    assert ids == refids
    assert len(ids) == 1
    assert not (user_ids & ids)



from dataclasses import dataclass, field
from typing import TYPE_CHECKING, Iterable, List

from sphinxcontrib.bibtex.style.template import reference, year

from . import BaseReferenceStyle, BracketStyle

if TYPE_CHECKING:
    from pybtex.richtext import BaseText
    from pybtex.style.template import Node


@dataclass
class ExtraYearReferenceStyle(BaseReferenceStyle):
    """Reference just by year."""

    #: Bracket style.
    bracket: BracketStyle = field(default_factory=BracketStyle)

    def role_names(self) -> Iterable[str]:
        return ["year", "yearpar"]

    def outer(self, role_name: str, children: List["BaseText"]) -> "Node":
        return self.bracket.outer(
            children,
            brackets="par" in role_name,
            capfirst=False,
        )

    def inner(self, role_name: str) -> "Node":
        return reference[year]



from test.common import latex_citation_refs, latex_citations

import pytest


@pytest.mark.sphinx("latex", testroot="latex_refs")
def test_latex_refs(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "test.tex").read_text(encoding="utf-8-sig")
    assert len(latex_citations().findall(output)) == 1
    assert len(latex_citation_refs().findall(output)) == 1
    match = latex_citations().search(output)
    match_ref = latex_citation_refs().search(output)
    assert match.group("label") == "Huy57"
    assert match.group("docname") == "index"
    assert "De ratiociniis in ludo aleæ." in match.group("text")
    assert match_ref.group("refid") == match.group("id_")
    assert match_ref.group("docname") == "index"


@pytest.mark.sphinx("latex", testroot="latex_multidoc")
def test_latex_multidoc(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "test.tex").read_text(encoding="utf-8-sig")
    assert len(latex_citations().findall(output)) == 1
    assert len(latex_citation_refs().findall(output)) == 1
    match = latex_citations().search(output)
    match_ref = latex_citation_refs().search(output)
    assert match.group("docname") == match_ref.group("docname") == "sources"
    assert match.group("id_") is not None
    assert match_ref.group("refid") == match.group("id_")


@pytest.mark.sphinx("latex", testroot="latex_multidoc_2")
def test_latex_multidoc_2(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "test.tex").read_text(encoding="utf-8-sig")
    assert len(latex_citations().findall(output)) == 1
    assert len(latex_citation_refs().findall(output)) == 1
    match = latex_citations().search(output)
    match_ref = latex_citation_refs().search(output)
    assert match.group("docname") == match_ref.group("docname") == "sources"
    assert match.group("id_") is not None
    assert match_ref.group("refid") == match.group("id_")



import re
import shutil
import time
from test.common import html_citations

import pytest
from pybtex.exceptions import PybtexError
from sphinx.errors import ExtensionError

status_up_to_date = "checking bibtex cache.*up to date"
status_out_of_date = "checking bibtex cache.*out of date"
status_parsing = "parsing bibtex file.*parsed [0-9]+ entries"


# Test that updates to the bibfile generate the correct result when
# Sphinx is run again.
@pytest.mark.sphinx("html", testroot="bibfiles_out_of_date")
def test_bibfiles_out_of_date(make_app, app_params) -> None:
    args, kwargs = app_params
    app = make_app(*args, **kwargs)
    app.build()
    status = app._status.getvalue()
    # not found, parsing
    assert re.search(status_up_to_date, status) is None
    assert re.search(status_out_of_date, status) is not None
    assert re.search(status_parsing, status) is not None
    output = (app.outdir / "index.html").read_text()
    assert html_citations(label="1", text=".*Akkerdju.*").search(output)
    assert html_citations(label="2", text=".*Bro.*").search(output)
    assert html_citations(label="3", text=".*Chap.*").search(output)
    assert html_citations(label="4", text=".*Dude.*").search(output)
    # wait to ensure different timestamp
    time.sleep(0.1)
    shutil.copyfile((app.srcdir / "test_new.xxx"), (app.srcdir / "test.bib"))
    app = make_app(*args, **kwargs)
    app.build()
    status = app._status.getvalue()
    # out of date, parsing
    assert re.search(status_up_to_date, status) is None
    assert re.search(status_out_of_date, status) is not None
    assert re.search(status_parsing, status) is not None
    output = (app.outdir / "index.html").read_text()
    assert html_citations(label="1", text=".*Eminence.*").search(output)
    assert html_citations(label="2", text=".*Frater.*").search(output)
    assert html_citations(label="3", text=".*Giggles.*").search(output)
    assert html_citations(label="4", text=".*Handy.*").search(output)
    # wait to ensure different timestamp
    time.sleep(0.1)
    shutil.copyfile((app.srcdir / "index_new.xxx"), (app.srcdir / "index.rst"))
    app = make_app(*args, **kwargs)
    app.build()
    status = app._status.getvalue()
    # up to date
    assert re.search(status_up_to_date, status) is not None
    assert re.search(status_out_of_date, status) is None
    assert re.search(status_parsing, status) is None


@pytest.mark.sphinx("html", testroot="bibfiles_not_found")
def test_bibfiles_not_found(app, warning) -> None:
    app.build()
    warning.seek(0)
    warnings = warning.readlines()
    assert len(warnings) == 2
    assert "could not open bibtex file" in warnings[0]
    assert "not found or not configured in bibtex_bibfiles" in warnings[1]


@pytest.mark.sphinx("html", testroot="bibfiles_missing_conf")
def test_bibfiles_missing_conf(make_app, app_params) -> None:
    args, kwargs = app_params
    with pytest.raises(ExtensionError, match="bibtex_bibfiles"):
        make_app(*args, **kwargs)


@pytest.mark.sphinx("html", testroot="bibfiles_encoding", freshenv=True)
def test_bibfiles_encoding(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.html").read_text(encoding="utf-8-sig")
    assert "Äpplé" in output


@pytest.mark.sphinx(
    "html",
    testroot="bibfiles_encoding",
    freshenv=True,
    confoverrides={"bibtex_encoding": "ascii"},
)
def test_bibfiles_encoding_bad(make_app, app_params) -> None:
    args, kwargs = app_params
    with pytest.raises(PybtexError, match="can't decode byte 0xc4"):
        make_app(*args, **kwargs)


@pytest.mark.sphinx(
    "html",
    testroot="bibfiles_encoding",
    freshenv=True,
    confoverrides={"bibtex_encoding": "invalid"},
)
def test_bibfiles_encoding_invalid(make_app, app_params) -> None:
    args, kwargs = app_params
    with pytest.raises(LookupError, match="unknown encoding"):
        make_app(*args, **kwargs)


@pytest.mark.sphinx("html", testroot="bibfiles_subfolder")
def test_bibfiles_subfolder(app, warning) -> None:
    app.build()
    assert not warning.getvalue()


@pytest.mark.sphinx("html", testroot="bibfiles_multiple_macros")
def test_bibfiles_multiple_macros(app, warning) -> None:
    app.build()
    assert not warning.getvalue()
    output = (app.outdir / "index.html").read_text()
    assert html_citations(label="1", text=r".*Rev\. Mod\. Phys\..*").search(output)


@pytest.mark.sphinx("html", testroot="bibfiles_multiple_keys")
def test_bibfiles_multiple_keys(app, warning) -> None:
    app.build()
    assert (
        re.search(
            "bibliography data error in .*: repeated bibliograhpy entry: test",
            warning.getvalue(),
        )
        is not None
    )
    output = (app.outdir / "index.html").read_text()
    assert html_citations(label="1", text=".*Test one.*").search(output)


# https://bitbucket.org/pybtex-devs/pybtex/issues/441/author-editor-fields-not-resolved-via
@pytest.mark.xfail(reason="pybtex bug")
@pytest.mark.sphinx("html", testroot="bibfiles_crossref")
def test_bibfiles_crossref(app, warning) -> None:
    app.build()
    assert not warning.getvalue()



import pybtex.plugin
from pybtex.style.formatting.unsrt import Style as UnsrtStyle
from pybtex.style.template import words

extensions = ["sphinxcontrib.bibtex"]
exclude_patterns = ["_build"]
bibtex_bibfiles = ["test.bib"]
bibtex_default_style = "nowebref"


class NoWebRefStyle(UnsrtStyle):
    def format_web_refs(self, e):
        # the following is just one simple way to return an empty node
        return words[""]


pybtex.plugin.register_plugin("pybtex.style.formatting", "nowebref", NoWebRefStyle)



extensions = ["rinoh.frontend.sphinx", "sphinxcontrib.bibtex"]
exclude_patterns = ["_build"]
bibtex_bibfiles = ["test.bib"]
rinoh_documents = [
    dict(doc="index", target="book", toctree_only=False, template="template.rtt"),
]



from pybtex.plugin import register_plugin
from pybtex.style.formatting.unsrt import Style as UnsrtStyle
from pybtex.style.labels.alpha import LabelStyle as AlphaLabelStyle

extensions = ["sphinxcontrib.bibtex"]
exclude_patterns = ["_build"]
bibtex_bibfiles = ["test.bib"]


class ApaLabelStyle(AlphaLabelStyle):
    def format_label(self, entry):
        return "APA"


class ApaStyle(UnsrtStyle):
    default_label_style = "apa"


register_plugin("pybtex.style.labels", "apa", ApaLabelStyle)
register_plugin("pybtex.style.formatting", "apastyle", ApaStyle)
